/*
 * Copyright (C) 2021 Anton Romanov - All Rights Reserved
 * You may use, distribute and modify this code, please write to: romanov73@gmail.com.
 */

package ru.ulstu.extractor.controller;

import org.springframework.data.domain.Page;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import ru.ulstu.extractor.model.Commit;
import ru.ulstu.extractor.model.OffsetablePageRequest;
import ru.ulstu.extractor.model.mvc.FilterForm;
import ru.ulstu.extractor.service.FilteringService;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static ru.ulstu.extractor.controller.Route.FILTER_COMMITS;

@Controller
public class GitFilteringController {
    private final static int DEFAULT_PAGE_SIZE = 20;
    private final FilteringService filteringService;

    public GitFilteringController(FilteringService filteringService) {
        this.filteringService = filteringService;
    }

    @RequestMapping(value = FILTER_COMMITS, method = RequestMethod.GET)
    public String listCommits(
            Model model,
            @ModelAttribute FilterForm filterForm,
            @RequestParam("page") Optional<Integer> page,
            @RequestParam("size") Optional<Integer> size,
            @RequestParam String repositoryUrl,
            @RequestParam String branchName) {
        int currentPage = page.orElse(1);
        int pageSize = size.orElse(DEFAULT_PAGE_SIZE);

        Page<Commit> commitsPage = filteringService.getCommits(repositoryUrl,
                branchName,
                new OffsetablePageRequest(currentPage - 1, pageSize));
        int totalPages = commitsPage.getTotalPages();
        if (totalPages > 0) {
            List<Integer> pageNumbers = IntStream.rangeClosed(1, totalPages)
                    .boxed()
                    .collect(Collectors.toList());
            model.addAttribute("pageNumbers", pageNumbers);
        }
        filterForm.setCommitsPage(commitsPage);
        filterForm.setBranch(branchName);
        filterForm.setUrl(repositoryUrl);
        model.addAttribute("filterForm", filterForm);
        model.addAttribute("authors", filteringService.getRepositoryAuthors(repositoryUrl, branchName));
        return FILTER_COMMITS;
    }
}
